@@ -1,18 +1,20 @@
 //! Validator for `format` keyword.
+use std::{net::IpAddr, str::FromStr};
+
+use fancy_regex::Regex;
+use serde_json::{Map, Value};
+use url::Url;
+use uuid::Uuid;
+
 use crate::{
     compilation::{context::CompilationContext, JSONSchema},
     error::{error, no_error, ErrorIterator, ValidationError},
     keywords::{pattern, CompilationResult},
     paths::{InstancePath, JSONPointer},
+    primitive_type::PrimitiveType,
     validator::Validate,
     Draft,
 };
-use chrono::{DateTime, NaiveDate};
-use fancy_regex::Regex;
-use serde_json::{Map, Value};
-
-use std::{net::IpAddr, str::FromStr};
-use url::Url;
 
 lazy_static::lazy_static! {
     static ref DATE_RE: Regex =
@@ -82,7 +84,12 @@ impl Validate for DateValidator {
     validate!("date");
     fn is_valid(&self, _: &JSONSchema, instance: &Value) -> bool {
         if let Value::String(item) = instance {
-            if NaiveDate::parse_from_str(item, "%Y-%m-%d").is_ok() {
+            if time::Date::parse(
+                item,
+                &time::macros::format_description!("[year]-[month]-[day]"),
+            )
+            .is_ok()
+            {
                 // Padding with zeroes is ignored by the underlying parser. The most efficient
                 // way to check it will be to use a custom parser that won't ignore zeroes,
                 // but this regex will do the trick and costs ~20% extra time in this validator.
@@ -102,7 +109,8 @@ impl Validate for DateTimeValidator {
     validate!("date-time");
     fn is_valid(&self, _: &JSONSchema, instance: &Value) -> bool {
         if let Value::String(item) = instance {
-            DateTime::parse_from_rfc3339(item).is_ok()
+            time::OffsetDateTime::parse(item, &time::format_description::well_known::Rfc3339)
+                .is_ok()
         } else {
             true
         }
@@ -158,11 +166,13 @@ impl Validate for HostnameValidator {
             !(item.ends_with('-')
                 || item.starts_with('-')
                 || item.is_empty()
-                || item.chars().count() > 255
+                || bytecount::num_chars(item.as_bytes()) > 255
                 || item
                     .chars()
                     .any(|c| !(c.is_alphanumeric() || c == '-' || c == '.'))
-                || item.split('.').any(|part| part.chars().count() > 63))
+                || item
+                    .split('.')
+                    .any(|part| bytecount::num_chars(part.as_bytes()) > 63))
         } else {
             true
         }
@@ -176,11 +186,13 @@ impl Validate for IDNHostnameValidator {
             !(item.ends_with('-')
                 || item.starts_with('-')
                 || item.is_empty()
-                || item.chars().count() > 255
+                || bytecount::num_chars(item.as_bytes()) > 255
                 || item
                     .chars()
                     .any(|c| !(c.is_alphanumeric() || c == '-' || c == '.'))
-                || item.split('.').any(|part| part.chars().count() > 63))
+                || item
+                    .split('.')
+                    .any(|part| bytecount::num_chars(part.as_bytes()) > 63))
         } else {
             true
         }
@@ -328,6 +340,30 @@ impl Validate for URITemplateValidator {
     }
 }
 
+format_validator!(UUIDValidator, "uuid");
+impl Validate for UUIDValidator {
+    validate!("uuid");
+    fn is_valid(&self, _: &JSONSchema, instance: &Value) -> bool {
+        if let Value::String(item) = instance {
+            Uuid::from_str(item.as_str()).is_ok()
+        } else {
+            true
+        }
+    }
+}
+
+format_validator!(DurationValidator, "duration");
+impl Validate for DurationValidator {
+    validate!("duration");
+    fn is_valid(&self, _: &JSONSchema, instance: &Value) -> bool {
+        if let Value::String(item) = instance {
+            iso8601::duration(item).is_ok()
+        } else {
+            true
+        }
+    }
+}
+
 struct CustomFormatValidator {
     schema_path: JSONPointer,
     format_name: &'static str,
@@ -388,6 +424,10 @@ pub(crate) fn compile<'a>(
     schema: &'a Value,
     context: &CompilationContext,
 ) -> Option<CompilationResult<'a>> {
+    if !context.config.validate_formats() {
+        return None;
+    }
+
     if let Value::String(format) = schema {
         if let Some((format, func)) = context.config.format(format) {
             return Some(CustomFormatValidator::compile(context, format, *func));
@@ -402,39 +442,79 @@ pub(crate) fn compile<'a>(
             "idn-hostname" if draft_version == Draft::Draft7 => {
                 Some(IDNHostnameValidator::compile(context))
             }
+            #[cfg(feature = "draft201909")]
+            "idn-hostname" if draft_version == Draft::Draft201909 => {
+                Some(IDNHostnameValidator::compile(context))
+            }
             "ipv4" => Some(IpV4Validator::compile(context)),
             "ipv6" => Some(IpV6Validator::compile(context)),
             "iri-reference" if draft_version == Draft::Draft7 => {
                 Some(IRIReferenceValidator::compile(context))
             }
+            #[cfg(feature = "draft201909")]
+            "iri-reference" if draft_version == Draft::Draft201909 => {
+                Some(IRIReferenceValidator::compile(context))
+            }
             "iri" if draft_version == Draft::Draft7 => Some(IRIValidator::compile(context)),
+            #[cfg(feature = "draft201909")]
+            "iri" if draft_version == Draft::Draft201909 => Some(IRIValidator::compile(context)),
             "json-pointer" if draft_version == Draft::Draft6 || draft_version == Draft::Draft7 => {
                 Some(JSONPointerValidator::compile(context))
             }
+            #[cfg(feature = "draft201909")]
+            "json-pointer" if draft_version == Draft::Draft201909 => {
+                Some(JSONPointerValidator::compile(context))
+            }
             "regex" => Some(RegexValidator::compile(context)),
             "relative-json-pointer" if draft_version == Draft::Draft7 => {
                 Some(RelativeJSONPointerValidator::compile(context))
             }
+            #[cfg(feature = "draft201909")]
+            "relative-json-pointer" if draft_version == Draft::Draft201909 => {
+                Some(RelativeJSONPointerValidator::compile(context))
+            }
             "time" => Some(TimeValidator::compile(context)),
             "uri-reference" if draft_version == Draft::Draft6 || draft_version == Draft::Draft7 => {
                 Some(URIReferenceValidator::compile(context))
             }
+            #[cfg(feature = "draft201909")]
+            "uri-reference" if draft_version == Draft::Draft201909 => {
+                Some(URIReferenceValidator::compile(context))
+            }
             "uri-template" if draft_version == Draft::Draft6 || draft_version == Draft::Draft7 => {
                 Some(URITemplateValidator::compile(context))
             }
+            #[cfg(feature = "draft201909")]
+            "uri-template" if draft_version == Draft::Draft201909 => {
+                Some(URITemplateValidator::compile(context))
+            }
+            #[cfg(feature = "draft201909")]
+            "uuid" if draft_version == Draft::Draft201909 => Some(UUIDValidator::compile(context)),
             "uri" => Some(URIValidator::compile(context)),
+            #[cfg(feature = "draft201909")]
+            "duration" if draft_version == Draft::Draft201909 => {
+                Some(DurationValidator::compile(context))
+            }
             _ => None,
         }
     } else {
-        Some(Err(ValidationError::schema(schema)))
+        Some(Err(ValidationError::single_type_error(
+            JSONPointer::default(),
+            context.clone().into_pointer(),
+            schema,
+            PrimitiveType::String,
+        )))
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::{compilation::JSONSchema, tests_util};
     use serde_json::json;
 
+    #[cfg(feature = "draft201909")]
+    use crate::schemas::Draft::Draft201909;
+    use crate::{compilation::JSONSchema, tests_util};
+
     #[test]
     fn ignored_format() {
         let schema = json!({"format": "custom", "type": "string"});
@@ -443,6 +523,27 @@ mod tests {
         assert!(compiled.is_valid(&instance))
     }
 
+    #[test]
+    fn format_validation() {
+        let schema = json!({"format": "email", "type": "string"});
+        let email_instance = json!("email@example.com");
+        let not_email_instance = json!("foo");
+
+        let with_validation = JSONSchema::options()
+            .should_validate_formats(true)
+            .compile(&schema)
+            .unwrap();
+        let without_validation = JSONSchema::options()
+            .should_validate_formats(false)
+            .compile(&schema)
+            .unwrap();
+
+        assert!(with_validation.is_valid(&email_instance));
+        assert!(!with_validation.is_valid(&not_email_instance));
+        assert!(without_validation.is_valid(&email_instance));
+        assert!(without_validation.is_valid(&not_email_instance));
+    }
+
     #[test]
     fn ecma_regex() {
         // See GH-230
@@ -456,4 +557,55 @@ mod tests {
     fn schema_path() {
         tests_util::assert_schema_path(&json!({"format": "date"}), &json!("bla"), "/format")
     }
+
+    #[cfg(feature = "draft201909")]
+    #[test]
+    fn uuid() {
+        let schema = json!({"format": "uuid", "type": "string"});
+
+        let passing_instance = json!("f308a72c-fa84-11eb-9a03-0242ac130003");
+        let failing_instance = json!("1");
+
+        let compiled = JSONSchema::options()
+            .with_draft(Draft201909)
+            .should_validate_formats(true)
+            .compile(&schema)
+            .unwrap();
+
+        assert!(compiled.is_valid(&passing_instance));
+        assert!(!compiled.is_valid(&failing_instance))
+    }
+
+    #[test]
+    fn uri() {
+        let schema = json!({"format": "uri", "type": "string"});
+
+        let passing_instance = json!("https://phillip.com");
+        let failing_instance = json!("redis");
+
+        tests_util::is_valid(&schema, &passing_instance);
+        tests_util::is_not_valid(&schema, &failing_instance);
+    }
+
+    #[cfg(feature = "draft201909")]
+    #[test]
+    fn duration() {
+        let schema = json!({"format": "duration", "type": "string"});
+
+        let passing_instances = vec![json!("P15DT1H22M1.5S"), json!("P30D"), json!("PT5M")];
+        let failing_instances = vec![json!("15DT1H22M1.5S"), json!("unknown")];
+
+        let compiled = JSONSchema::options()
+            .with_draft(Draft201909)
+            .should_validate_formats(true)
+            .compile(&schema)
+            .unwrap();
+
+        for passing_instance in passing_instances {
+            assert!(compiled.is_valid(&passing_instance));
+        }
+        for failing_instance in failing_instances {
+            assert!(!compiled.is_valid(&failing_instance));
+        }
+    }
 }
