@@ -3,6 +3,7 @@ use crate::{
     error::{error, no_error, ErrorIterator, ValidationError},
     keywords::CompilationResult,
     paths::{InstancePath, JSONPointer},
+    primitive_type::PrimitiveType,
     validator::Validate,
 };
 use num_cmp::NumCmp;
@@ -10,14 +11,17 @@ use serde_json::{Map, Value};
 
 pub(crate) struct MaximumU64Validator {
     limit: u64,
+    limit_val: Value,
     schema_path: JSONPointer,
 }
 pub(crate) struct MaximumI64Validator {
     limit: i64,
+    limit_val: Value,
     schema_path: JSONPointer,
 }
 pub(crate) struct MaximumF64Validator {
     limit: f64,
+    limit_val: Value,
     schema_path: JSONPointer,
 }
 
@@ -37,7 +41,7 @@ macro_rules! validate {
                         self.schema_path.clone(),
                         instance_path.into(),
                         instance,
-                        self.limit as f64,
+                        self.limit_val.clone(),
                     )) // do not cast
                 }
             }
@@ -95,7 +99,7 @@ impl Validate for MaximumF64Validator {
                 self.schema_path.clone(),
                 instance_path.into(),
                 instance,
-                self.limit,
+                self.limit_val.clone(),
             ))
         }
     }
@@ -115,15 +119,32 @@ pub(crate) fn compile<'a>(
     if let Value::Number(limit) = schema {
         let schema_path = context.as_pointer_with("maximum");
         if let Some(limit) = limit.as_u64() {
-            Some(Ok(Box::new(MaximumU64Validator { limit, schema_path })))
+            Some(Ok(Box::new(MaximumU64Validator {
+                limit,
+                limit_val: schema.clone(),
+                schema_path,
+            })))
         } else if let Some(limit) = limit.as_i64() {
-            Some(Ok(Box::new(MaximumI64Validator { limit, schema_path })))
+            Some(Ok(Box::new(MaximumI64Validator {
+                limit,
+                limit_val: schema.clone(),
+                schema_path,
+            })))
         } else {
             let limit = limit.as_f64().expect("Always valid");
-            Some(Ok(Box::new(MaximumF64Validator { limit, schema_path })))
+            Some(Ok(Box::new(MaximumF64Validator {
+                limit,
+                limit_val: schema.clone(),
+                schema_path,
+            })))
         }
     } else {
-        Some(Err(ValidationError::schema(schema)))
+        Some(Err(ValidationError::single_type_error(
+            JSONPointer::default(),
+            context.clone().into_pointer(),
+            schema,
+            PrimitiveType::Number,
+        )))
     }
 }
 
