@@ -122,7 +122,10 @@ mod tests {
     #[test_case(&json!({"uniqueItems": true}), "uniqueItems: true")]
     fn debug_representation(schema: &Value, expected: &str) {
         let compiled = JSONSchema::compile(schema).unwrap();
-        assert_eq!(format!("{:?}", compiled.validators[0]), expected);
+        assert_eq!(
+            format!("{:?}", compiled.node.validators().next().unwrap()),
+            expected
+        );
     }
 
     #[test_case(&json!({"items": [{}], "additionalItems": {"type": "integer"}}), &json!([ null, 2, 3, "foo" ]), r#""foo" is not of type "integer""#)]
@@ -133,10 +136,15 @@ mod tests {
     #[test_case(&json!({"const": 2}), &json!(5), r#"2 was expected"#)]
     #[test_case(&json!({"contains": {"minimum": 5}}), &json!([2, 3, 4]), r#"None of [2,3,4] are valid under the given schema"#)]
     #[test_case(&json!({"enum": [1, 2, 3]}), &json!(4), r#"4 is not one of [1,2,3]"#)]
-    #[test_case(&json!({"exclusiveMaximum": 3.0}), &json!(3.0), r#"3.0 is greater than or equal to the maximum of 3"#)]
-    #[test_case(&json!({"exclusiveMinimum": 1.1}), &json!(1.1), r#"1.1 is less than or equal to the minimum of 1.1"#)]
+    #[test_case(&json!({"exclusiveMaximum": 3}), &json!(3.0), r#"3.0 is greater than or equal to the maximum of 3"#)]
+    #[test_case(&json!({"exclusiveMaximum": 3.0}), &json!(3.0), r#"3.0 is greater than or equal to the maximum of 3.0"#)]
+    #[test_case(&json!({"exclusiveMinimum": 1}), &json!(1.0), r#"1.0 is less than or equal to the minimum of 1"#)]
+    #[test_case(&json!({"exclusiveMinimum": 1.0}), &json!(1), r#"1 is less than or equal to the minimum of 1.0"#)]
     #[test_case(&json!({"format": "ipv4"}), &json!("2001:0db8:85a3:0000:0000:8a2e:0370:7334"), r#""2001:0db8:85a3:0000:0000:8a2e:0370:7334" is not a "ipv4""#)]
-    #[test_case(&json!({"maximum": 3.0}), &json!(3.5), r#"3.5 is greater than the maximum of 3"#)]
+    #[test_case(&json!({"maximum": 3}), &json!(3.5), r#"3.5 is greater than the maximum of 3"#)]
+    #[test_case(&json!({"maximum": 3.0}), &json!(3.5), r#"3.5 is greater than the maximum of 3.0"#)]
+    #[test_case(&json!({"minimum": 3}), &json!(2.5), r#"2.5 is less than the minimum of 3"#)]
+    #[test_case(&json!({"minimum": 3.0}), &json!(2.5), r#"2.5 is less than the minimum of 3.0"#)]
     #[test_case(&json!({"maxItems": 2}), &json!([1, 2, 3]), r#"[1,2,3] has more than 2 items"#)]
     #[test_case(&json!({"maxLength": 2}), &json!("foo"), r#""foo" is longer than 2 characters"#)]
     #[test_case(&json!({"maxProperties": 2}), &json!({"foo": 1, "bar": 2, "baz": 3}), r#"{"bar":2,"baz":3,"foo":1} has more than 2 properties"#)]
