@@ -2,14 +2,17 @@ use crate::{
     compilation::{compile_validators, context::CompilationContext, JSONSchema},
     error::{no_error, ErrorIterator, ValidationError},
     keywords::CompilationResult,
-    paths::InstancePath,
-    validator::{format_validators, Validate, Validators},
+    output::BasicOutput,
+    paths::{InstancePath, JSONPointer},
+    primitive_type::PrimitiveType,
+    schema_node::SchemaNode,
+    validator::{format_validators, PartialApplication, Validate},
 };
 use fancy_regex::Regex;
 use serde_json::{Map, Value};
 
 pub(crate) struct PatternPropertiesValidator {
-    patterns: Vec<(Regex, Validators)>,
+    patterns: Vec<(Regex, SchemaNode)>,
 }
 
 impl PatternPropertiesValidator {
@@ -25,7 +28,14 @@ impl PatternPropertiesValidator {
             patterns.push((
                 match Regex::new(pattern) {
                     Ok(r) => r,
-                    Err(_) => return Err(ValidationError::schema(subschema)),
+                    Err(_) => {
+                        return Err(ValidationError::format(
+                            JSONPointer::default(),
+                            keyword_context.clone().into_pointer(),
+                            subschema,
+                            "regex",
+                        ))
+                    }
                 },
                 compile_validators(subschema, &pattern_context)?,
             ));
@@ -37,20 +47,17 @@ impl PatternPropertiesValidator {
 impl Validate for PatternPropertiesValidator {
     fn is_valid(&self, schema: &JSONSchema, instance: &Value) -> bool {
         if let Value::Object(item) = instance {
-            self.patterns.iter().all(move |(re, validators)| {
+            self.patterns.iter().all(move |(re, node)| {
                 item.iter()
                     .filter(move |(key, _)| re.is_match(key).unwrap_or(false))
-                    .all(move |(_key, value)| {
-                        validators
-                            .iter()
-                            .all(move |validator| validator.is_valid(schema, value))
-                    })
+                    .all(move |(_key, value)| node.is_valid(schema, value))
             })
         } else {
             true
         }
     }
 
+    #[allow(clippy::needless_collect)]
     fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
@@ -61,14 +68,12 @@ impl Validate for PatternPropertiesValidator {
             let errors: Vec<_> = self
                 .patterns
                 .iter()
-                .flat_map(move |(re, validators)| {
+                .flat_map(move |(re, node)| {
                     item.iter()
                         .filter(move |(key, _)| re.is_match(key).unwrap_or(false))
                         .flat_map(move |(key, value)| {
                             let instance_path = instance_path.push(key.clone());
-                            validators.iter().flat_map(move |validator| {
-                                validator.validate(schema, value, &instance_path)
-                            })
+                            node.validate(schema, value, &instance_path)
                         })
                 })
                 .collect();
@@ -77,6 +82,32 @@ impl Validate for PatternPropertiesValidator {
             no_error()
         }
     }
+
+    fn apply<'a>(
+        &'a self,
+        schema: &JSONSchema,
+        instance: &Value,
+        instance_path: &InstancePath,
+    ) -> PartialApplication<'a> {
+        if let Value::Object(item) = instance {
+            let mut matched_propnames = Vec::with_capacity(item.len());
+            let mut sub_results = BasicOutput::default();
+            for (pattern, node) in &self.patterns {
+                for (key, value) in item {
+                    if pattern.is_match(key).unwrap_or(false) {
+                        let path = instance_path.push(key.clone());
+                        matched_propnames.push(key.clone());
+                        sub_results += node.apply_rooted(schema, value, &path);
+                    }
+                }
+            }
+            let mut result: PartialApplication = sub_results.into();
+            result.annotate(serde_json::Value::from(matched_propnames).into());
+            result
+        } else {
+            PartialApplication::valid_empty()
+        }
+    }
 }
 
 impl core::fmt::Display for PatternPropertiesValidator {
@@ -86,7 +117,7 @@ impl core::fmt::Display for PatternPropertiesValidator {
             "patternProperties: {{{}}}",
             self.patterns
                 .iter()
-                .map(|(key, validators)| { format!("{}: {}", key, format_validators(validators)) })
+                .map(|(key, node)| { format!("{}: {}", key, format_validators(node.validators())) })
                 .collect::<Vec<String>>()
                 .join(", ")
         )
@@ -95,7 +126,7 @@ impl core::fmt::Display for PatternPropertiesValidator {
 
 pub(crate) struct SingleValuePatternPropertiesValidator {
     pattern: Regex,
-    validators: Validators,
+    node: SchemaNode,
 }
 
 impl SingleValuePatternPropertiesValidator {
@@ -110,9 +141,16 @@ impl SingleValuePatternPropertiesValidator {
         Ok(Box::new(SingleValuePatternPropertiesValidator {
             pattern: match Regex::new(pattern) {
                 Ok(r) => r,
-                Err(_) => return Err(ValidationError::schema(schema)),
+                Err(_) => {
+                    return Err(ValidationError::format(
+                        JSONPointer::default(),
+                        keyword_context.clone().into_pointer(),
+                        schema,
+                        "regex",
+                    ))
+                }
             },
-            validators: compile_validators(schema, &pattern_context)?,
+            node: compile_validators(schema, &pattern_context)?,
         }))
     }
 }
@@ -122,16 +160,13 @@ impl Validate for SingleValuePatternPropertiesValidator {
         if let Value::Object(item) = instance {
             item.iter()
                 .filter(move |(key, _)| self.pattern.is_match(key).unwrap_or(false))
-                .all(move |(_key, value)| {
-                    self.validators
-                        .iter()
-                        .all(move |validator| validator.is_valid(schema, value))
-                })
+                .all(move |(_key, value)| self.node.is_valid(schema, value))
         } else {
             true
         }
     }
 
+    #[allow(clippy::needless_collect)]
     fn validate<'a, 'b>(
         &self,
         schema: &'a JSONSchema,
@@ -144,9 +179,7 @@ impl Validate for SingleValuePatternPropertiesValidator {
                 .filter(move |(key, _)| self.pattern.is_match(key).unwrap_or(false))
                 .flat_map(move |(key, value)| {
                     let instance_path = instance_path.push(key.clone());
-                    self.validators.iter().flat_map(move |validator| {
-                        validator.validate(schema, value, &instance_path)
-                    })
+                    self.node.validate(schema, value, &instance_path)
                 })
                 .collect();
             Box::new(errors.into_iter())
@@ -154,6 +187,30 @@ impl Validate for SingleValuePatternPropertiesValidator {
             no_error()
         }
     }
+
+    fn apply<'a>(
+        &'a self,
+        schema: &JSONSchema,
+        instance: &Value,
+        instance_path: &InstancePath,
+    ) -> PartialApplication<'a> {
+        if let Value::Object(item) = instance {
+            let mut matched_propnames = Vec::with_capacity(item.len());
+            let mut outputs = BasicOutput::default();
+            for (key, value) in item {
+                if self.pattern.is_match(key).unwrap_or(false) {
+                    let path = instance_path.push(key.clone());
+                    matched_propnames.push(key.clone());
+                    outputs += self.node.apply_rooted(schema, value, &path);
+                }
+            }
+            let mut result: PartialApplication = outputs.into();
+            result.annotate(serde_json::Value::from(matched_propnames).into());
+            result
+        } else {
+            PartialApplication::valid_empty()
+        }
+    }
 }
 
 impl core::fmt::Display for SingleValuePatternPropertiesValidator {
@@ -162,7 +219,7 @@ impl core::fmt::Display for SingleValuePatternPropertiesValidator {
             f,
             "patternProperties: {{{}: {}}}",
             self.pattern,
-            format_validators(&self.validators)
+            format_validators(self.node.validators())
         )
     }
 }
@@ -187,7 +244,12 @@ pub(crate) fn compile<'a>(
                     Some(PatternPropertiesValidator::compile(map, context))
                 }
             } else {
-                Some(Err(ValidationError::schema(schema)))
+                Some(Err(ValidationError::single_type_error(
+                    JSONPointer::default(),
+                    context.clone().into_pointer(),
+                    schema,
+                    PrimitiveType::Object,
+                )))
             }
         }
     }
