@@ -7,6 +7,8 @@
 //!   - JSON Schema drafts 4, 6, 7 (except some optional test cases);
 //!   - Loading remote documents via HTTP(S);
 //!
+//! This library is functional and ready for use, but its API is still evolving to the 1.0 API.
+//!
 //! ## Usage Examples:
 //! A schema can be compiled with two main flavours:
 //!  * using default configurations
@@ -75,16 +77,23 @@
     unreachable_pub,
     variant_size_differences
 )]
-#![allow(clippy::unnecessary_wraps, clippy::upper_case_acronyms)]
+#![allow(
+    clippy::unnecessary_wraps,
+    clippy::upper_case_acronyms,
+    clippy::needless_collect
+)]
 #![cfg_attr(not(test), allow(clippy::integer_arithmetic, clippy::unwrap_used))]
 mod compilation;
 mod content_encoding;
 mod content_media_type;
 pub mod error;
 mod keywords;
+mod output;
+pub use output::{BasicOutput, Output};
 pub mod paths;
 pub mod primitive_type;
 mod resolver;
+mod schema_node;
 mod schemas;
 mod validator;
 
@@ -185,8 +194,9 @@ pub(crate) mod tests_util {
 
 #[cfg(test)]
 mod tests {
-    use super::is_valid;
+    use super::{is_valid, Draft, JSONSchema};
     use serde_json::json;
+    use test_case::test_case;
 
     #[test]
     fn test_is_valid() {
@@ -196,4 +206,24 @@ mod tests {
         assert!(is_valid(&schema, &valid));
         assert!(!is_valid(&schema, &invalid));
     }
+
+    #[test_case(Draft::Draft4)]
+    #[test_case(Draft::Draft6)]
+    #[test_case(Draft::Draft7)]
+    fn meta_schemas(draft: Draft) {
+        // See GH-258
+        for schema in [json!({"enum": [0, 0.0]}), json!({"enum": []})] {
+            assert!(JSONSchema::options()
+                .with_draft(draft)
+                .compile(&schema)
+                .is_ok())
+        }
+    }
+
+    #[test]
+    fn incomplete_escape_in_pattern() {
+        // See GH-253
+        let schema = json!({"pattern": "\\u"});
+        assert!(JSONSchema::compile(&schema).is_err())
+    }
 }
